#target photoshop
//
// InstallCSX.jsx
//
// $Id: InstallCSX.jsx,v 1.21 2008/12/22 01:13:38 xbytor Exp $
// Copyright: (c)2008, xbytor
// License: http://creativecommons.org/licenses/LGPL/2.1
// Contact: xbytor@gmail.com
//
CSX_RELEASE = "v1.4.2";

isCS5 = function()  { return app.version.match(/^12\./); };
isCS4 = function()  { return app.version.match(/^11\./); };
isCS3 = function()  { return app.version.match(/^10\./); };
isCS2 = function()  { return app.version.match(/^9\./); };

isCSXCompatible = function() {
  return isCS2() || isCS3() || isCS4();
};

isWindows = function() {
  return $.os.match(/windows/i);
};
isMac = function() {
  return !isWindows();
};
isVista = function() {
  return $.os.match(/vista/i);
};
isVista64 = function() {
  return $.os.match(/vista\/64/i);
};
isPhotoshop = function() {
  return app.name.match(/photoshop/i);
};
listProps = function(obj) {
  var s = '';
  for (var x in obj) {
    s += x + ":\t";
    try {
      var o = obj[x];
      s += (typeof o == "function") ? "[function]" : o;
    } catch (e) {
    }
    s += "\r\n";
  }
  return s;
};
Xstdlib = function() {};       // Let's borrow some stuff....
Xstdlib.logStr = '';
Xstdlib.log = function(msg) {
  var file;

  if (!Xstdlib.log.enabled) {
    return;
  }

  if (!Xstdlib.log.filename) {
    return;
  }

  if (!Xstdlib.log.fptr) {
    file = new File(Xstdlib.log.filename);
    //if (file.exists) file.remove();
    if (!file.open("w")) {
      Error.runtimeError(19, "Unable to open log file " + file + ": "
	                     + file.error);
    }
    file.seek(0, 0); // jump to the end of the file
    if (isMac()) {
      file.lineFeed = "unix";
    }
    Xstdlib.log.fptr = file;

  } else {
    file = Xstdlib.log.fptr;
    if (!file.open("e"))  {
      Error.runtimeError(19, "Unable to open log file " + file + ": "
	                     + file.error);
    }
    file.seek(0, 2); // jump to the end of the file
  }
  
  Xstdlib.logStr += msg + '\r\n';
  if (!file.writeln(new Date() + " - " + msg)) {
    Error.runtimeError(19, "Unable to write to log file " + file + ": "
                       + file.error);
  }
  
   file.close();
};
Xstdlib.log.filename = undefined; //"~/logfile.txt";
Xstdlib.log.enabled = true;

Xstdlib.exceptionMessage = function(e) {
  var str = '';
  var fname = (!e.fileName ? '???' : decodeURI(e.fileName));
  str += "   Message: " + e.message + '\n';
  str += "   File: " + fname + '\n';
  str += "   Line: " + (e.line || '???') + '\n';
  str += "   Error Name: " + e.name + '\n';
  str += "   Error Number: " + e.number + '\n';

  if (e.source) {
    var srcArray = e.source.split("\n");
    var a = e.line - 10;
    var b = e.line + 10;
    var c = e.line - 1;
    if (a < 0) {
      a = 0;
    }
    if (b > srcArray.length) {
      b = srcArray.length;
    }
    for ( var i = a; i < b; i++ ) {
      if ( i == c ) {
        str += "   Line: (" + (i + 1) + ") >> " + srcArray[i] + '\n';
      } else {
        str += "   Line: (" + (i + 1) + ")    " + srcArray[i] + '\n';
      }
    }
  }

  if ($.stack) {
    str += '\n' + $.stack + '\n';
  }

  return str;
};


Xstdlib.PRESETS_FOLDER = 
  new Folder(app.path + '/' +
             localize("$$$/ApplicationPresetsFolder/Presets=Presets"));
Xstdlib.SCRIPTS_FOLDER =
  new Folder(app.path + '/' +
             localize("$$$/ScriptingSupport/InstalledScripts=Presets/Scripts"));


Options = new Object();
Options.testMode = false;

Options.psScriptsFolder = Xstdlib.SCRIPTS_FOLDER;
Options.brScriptsFolder = undefined; // created in checkConfig
Options.xtoolsDataFolder = new Folder(Folder.userData + '/xtools');


InstallCSX = function() {
};
InstallCSX.run = function() {
  var inst = new InstallCSX();
  var opts = Options;

  return inst.install(opts);
};
InstallCSX.prototype.install = function(opts) {
  var self = this;

  // Check the package folder and the Photoshop and Bridge folders for sanity
  if (!self.checkConfig(opts)) {
    return false;
  }

  var cstr = ("CSX " + CSX_RELEASE + " will be installed at:\r\n\t" +
              decodeURI(opts.psScriptsFolder.fsName) +
              "\r\n\r\nand (for Bridge support) at:\r\n\t" +
              decodeURI(opts.brScriptsFolder.fsName) + "\r\n\r\n" +
              "Any previous versions should be removed prior to " +
              "installing this package.\r\n\r\n" +
              "Do you wish to continue?");

  if (!confirm(cstr)) {
    return false;
  }

  //========================= CSX ==================================

  Xstdlib.log("\r\nInstalling CSX Script");
  // Try to remove any old CSX scripts
  var oldScripts = opts.psScriptsFolder.getFiles("CSX-1_*.jsx");
  for (var i = 0; i < oldScripts.length; i++) {
    if (!oldScripts[i].remove()) {
      msg = ("*** Unable to remove old CSX Photoshop script: " +
             decodeURI(oldScripts[i]) + "\r\n" +
             "You may need to remove this script manually " +
             "for CSX to work correctly.");
      Xstdlib.log(msg);
      alert(msg);
    } else {
      Xstdlib.log("Removed " + decodeURI(oldScripts[i]));
    }
  }

  // Copy the primary CSX script
  if (!self.copyFile(opts.csxScript, opts.psScriptsFolder)) {
    return false;
  }

  //========================= CSX Support ==================================

  Xstdlib.log("\r\nInstalling CSX support files");

  // Create the new csx folder and copy over its contents
  Xstdlib.log("\r\nCreating csx installation directories");
  var csxdir = self.checkSubdir(opts.psScriptsFolder, "csx");
  if (!csxdir) {
    return false;
  }
  Xstdlib.log("\"csx\" created");

  Xstdlib.log("\r\nCopying csx/ files");
  if (!self.copyFiles(opts.csxFolder, csxdir)) {
    return false;
  }
  Xstdlib.log("csx/ files copied");

  //========================= Templates ==================================

  Xstdlib.log("\r\nInstalling template and other files");
  var templatesdir = self.checkSubdir(csxdir, "templates");
  if (!templatesdir) {
    return;
  }
  Xstdlib.log("\"csx/templates\" created");
  Xstdlib.log("Copying csx/templates files");
  if (!self.copyFiles(new Folder(opts.csxFolder + "/templates"),
                      templatesdir)) {
    return false;
  }
  Xstdlib.log("csx/templates files copied");


  //========================= Bridge ==================================

  Xstdlib.log("\r\nInstalling CSX Bridge files");

  // Create the new Bridge folders and copy over the Bridge script
  var brcsxdir = self.checkSubdir(csxdir, "Bridge Scripts Only");
  if (!brcsxdir) {
    return false;
  }
  Xstdlib.log("\"csx/Bridge Scripts Only\" created");

  // Delete any old versions
  var brfiles = brcsxdir.getFiles("CSXBridge*.jsx");
  if (brfiles.length > 0) {
    for (var i = 0; i < brfiles.length; i++) {
      if (!brfiles[i].remove()) {
        msg = ("*** Unable to remove old CSX Bridge script: " +
               decodeURI(brfiles[i]) + "\r\n" +
               "You may need to remove this script manually " +
               "for CSX does to work correctly.");
        Xstdlib.log(msg);
        alert(msg);
      } else {
        Xstdlib.log("Removed " + decodeURI(brfiles[i]));
      }
    }
  }

  Xstdlib.log("Copying csx/bridge files");
  if (!self.copyFiles(new Folder(opts.csxFolder + "/Bridge Scripts Only"),
                      brcsxdir)) {
    return false;
  }

  // Delete any old versions
  var brfiles = opts.brScriptsFolder.getFiles("CSXBridge*.jsx");
  if (brfiles.length > 0) {
    for (var i = 0; i < brfiles.length; i++) {
      if (!brfiles[i].remove()) {
        msg = ("*** Unable to remove old CSX Bridge script: " +
               decodeURI(brfiles[i]) + "\r\n" +
               "You may need to remove this script manually " +
               "for CSX does to work correctly.");
        Xstdlib.log(msg);
        alert(msg);
      } else {
        Xstdlib.log("Removed " + decodeURI(brfiles[i]));
      }
    }
  }

  var brfiles = brcsxdir.getFiles("CSXBridge*.jsx");
  if (brfiles.length == 0) {
    Error.runtimeError("CSXBridge file not found");
  }

  if (!self.copyFile(brfiles[0], opts.brScriptsFolder)) {
    return false;
  }
  Xstdlib.log("csx/bridge files copied");

  return true;
};

InstallCSX.prototype.error = function(msg) {
  var emsg = "Error: " + msg;

  Xstdlib.log(emsg);
  alert(emsg);
  return false;
};

InstallCSX.prototype.checkConfig = function(opts) {
  var self = this;

  Xstdlib.log("Checking configuration...");
  Xstdlib.log("Checking for Photoshop");
  if (!isPhotoshop()) {
    return self.error("Please run this script from Photoshop.");
  }
  Xstdlib.log("Photoshop good!\r\n");

  Xstdlib.log("Checking for CS2/CS3/CS4");
  if (!isCSXCompatible()) {
    return self.error("ContactSheetX is compatible with only " +
                      "Photoshop CS2/CS3/CS4.");
  }
  Xstdlib.log("CS2/CS3/CS4 good!\r\n");

  Xstdlib.log("Checking for Photoshop Scripts folder: " +
             decodeURI(opts.psScriptsFolder));
  if (!opts.psScriptsFolder.exists) {
    return self.error("Photoshop Scripts folder not found.\r\nExpected: " +
                      opts.psScriptsFolder);
  }
  Xstdlib.log("Photoshop Scripts folder good!\r\n");

  var brRoot = Folder.commonFiles.absoluteURI + "/Adobe";
  var brCS2 = brRoot + "/StartupScripts";
  var brCS3 = brRoot + "/Startup Scripts CS3/Adobe Photoshop";
  var brCS4 = brRoot + "/Startup Scripts CS4/Adobe Photoshop";
  if (isCS2()) {
    opts.brScriptsFolder = new Folder(brCS2);

  } else if (isCS3()) {
    opts.brScriptsFolder = new Folder(brCS3);

  } else if (isCS4()) {
    opts.brScriptsFolder = new Folder(brCS4);
  }

  Xstdlib.log("Checking for Bridge Scripts folder: " +
             decodeURI(opts.brScriptsFolder));
  if (!opts.brScriptsFolder.exists) {
    if (!isVista64()) {
      return self.error("Bridge Scripts folder not found.\r\nExpected: " +
                        opts.brScriptsFolder);
    }

    var v32path = BridgeTalk.getAppPath("photoshop-11.032");
    if (!v32path) {
      return self.error("Unable to install on 64bit Vista at this time." +
                        "Please follow the instructions for " +
                        "a manual installation");
    }
    var v32app = Folder(v32path);
    var psFolder = v32app.parent;
    var adobeFolder = psFolder.parent;
    var programFilesFolder = adobeFolder.parent;
    var brRoot = programFilesFolder.absoluteURI + "/Common Files/Adobe";
    var brCS4 = brRoot + "/Startup Scripts CS4/Adobe Photoshop";
    opts.brScriptsFolder = new Folder(brCS4);

    Xstdlib.log("Checking for Vista32 Bridge Scripts folder: " +
                decodeURI(opts.brScriptsFolder));

    if (!opts.brScriptsFolder.exists) {
      return self.error("Bridge Scripts folder not found.\r\nExpected: " +
                        opts.brScriptsFolder);
    }
  }
  Xstdlib.log("Bridge Scripts folder good!\r\n");

  Xstdlib.log("Checking for xtools preferences folder: " +
             decodeURI(opts.xtoolsDataFolder));
  if (!opts.xtoolsDataFolder.exists) {
    if (!opts.xtoolsDataFolder.create()) {
      return self.error("Unable to create preferences folder:\r\n" +
                        opts.xtoolsDataFolder + "\r\n" +
                        opts.xtoolsDataFolder.error);
    }
  }
  Xstdlib.log("xtools preferences folder good!\r\n");

  Xstdlib.log("Checking installation package");
  opts.sourceFolder = InstallCSX.getScriptFolder();
  opts.csxFolder = new Folder(opts.sourceFolder + "/csx");
  if (!opts.csxFolder.exists) {
    return self.error("Bad installation package, folder not found:\r\n" +
                      decodeURI(opts.csxFolder));
  }
  var files = opts.sourceFolder.getFiles("CSX-1_*.jsx");
  if (files.length == 0) {
    return self.error("Bad installation package, CSX Script not found");
  }
  opts.csxScript = files[0];

  Xstdlib.log("CSX Script found:\r\n" + decodeURI(opts.csxScript) + "\r\n");

  return true;
};
InstallCSX.getScriptFolder = function() { 
  return InstallCSX.getScriptFile().parent;
};
InstallCSX.getScriptFile = function() {
  if (!isCSXCompatible()) {
    return undefined;
  }

  var dbLevel = $.level;
  $.level = 0;
  var path = '';

  try {
    some_undefined_variable;
  } catch (e) {
    path = e.fileName;
  }

  $.level = dbLevel;

  return new File(path);
};

InstallCSX.prototype.checkSubdir = function(folder, sub, label) {
  var subdir = new Folder(folder + '/' + sub);

  if (!subdir.exists) {
    if (!subdir.create()) {
      self.error("Unable to create " + label + " folder:\r\n" +
                 subdir + "\r\n" +
                 subdir.error);

      return undefined;
    }
  }

  return subdir;
};

InstallCSX.prototype.copyFile = function(srcFile, target) {
  var self = this;

  if (!(target instanceof Folder)) {
    Error.runtimeError(9002, "Internal Error: copyFile(" +
	                   srcFile + ", " + target + ")");
  }

  if (!target.exists) {
    Error.runtimeError(9002, "The folder " + decodeURI(target.fsName) +
	                   " does not exist.");
  }

  var testRO = new File(target + '/csx-test.tmp');
  if (!testRO.open("w")) {
    var msg = ("The folder " + decodeURI(target.fsName) +
              " is 'readonly'. Please either invoke Photoshop " +
              "with Administrator permissions for CSX installation " +
              "(use 'Run as adminstrator' on XP/Vista) or change the " +
              "permissions on the folder to allow CSX to write files to " +
              "that folder.\r\n\r\n");
     var emsg = Xstdlib.exceptionMessage(new IOError()) + "\r\n\r\n" + msg;
	 Xstdlib.log(emsg);
	 alert(msg);
	 return false;
	 
  } else {
    testRO.close();
	testRO.remove();
  }

  Xstdlib.log("Copy file from " + decodeURI(srcFile) + " to " +
             decodeURI(target));
  var file = new File(target + "/" + srcFile.name);
  if (!srcFile.copy(file)) {
    var msg = "Copy failed: " + srcFile.error + "\r\n" + decodeURI(srcFile);
    if (file.exists) {
      msg += ("\r\n\r\nAn existing copy of this file exists and must be " +
              "removed manually before CSX can be installed.");
    }
    return self.error(msg);
  }
  return true;
};

InstallCSX.prototype.copyFiles = function(src, target) {
  var self = this;

  Xstdlib.log("\r\nCopying files from " + decodeURI(src) + " to " +
             decodeURI(target));

  var files = src.getFiles();
  for (var i = 0; i < files.length; i++) {
    var file = files[i];
    if (file instanceof Folder) {
      continue;
    }
    if (!self.copyFile(file, target)) {
      return false;
    }
  }
  return true;
};

InstallCSX.popup = function(title, str) {
  var w = new Window('dialog', title, [200,200,1000,750]);
  var t = w.add('edittext', [10,10,790,490], '',
                {multiline:true, readonly:true});
  t.text = str;
  w.add('button', [300,500,350,520], 'OK');
  w.show();
};


InstallCSX.main = function() {
  Xstdlib.log.filename = "~/InstallCSX.log";
  Xstdlib.log.enabled = true;

  var rc = false;
  Xstdlib.log("Start");
  Xstdlib.log("Version: " + CSX_RELEASE);
  Xstdlib.log("Revision: $Revision: 1.21 $");
  Xstdlib.log("App: " + app.name);
  Xstdlib.log("App Version: " + app.version);
  Xstdlib.log("OS: " + $.os);

  try {
    rc = InstallCSX.run();

  } catch (e) {
    var msg = (Xstdlib.exceptionMessage(e) + "\"\r\n" +
               "Installation terminated.");
    Xstdlib.log(msg);
    alert(msg);
  }

  Xstdlib.log("\r\nCSX Installation completed status: " +
             (rc ? "good." : "*** failed ***"));
  Xstdlib.log("Stop");

  if (confirm("Installation " + (rc ? "succeeded!" : "failed!") +
              "\r\n\r\nView log?")) {

    InstallCSX.popup("CSX Installation: " + Xstdlib.log.filename,
                     Xstdlib.logStr);
  }

  if (rc) {
    alert("You will need to restart Photoshop and Bridge to enable CSX.");
  }
};

InstallCSX.main();

"InstallCSX.jsx";
// EOF
