#!/bin/bash
#
# hp2mac.sh
#
# $Id: hp2mac.sh,v 1.2 2006/11/22 15:33:47 xbytor Exp $
#
# This is the Mac exec installation script.
# This creates the necessary files and folders needed to use HP2 mode
# under OSX.
#
# Installation instructions:
#
# This script will only operate on CSX rev1.26 or later.
#
# Before installation, CSX must be installed in your CS2 PresetsScripts folder.
# Normally, that folder is /Applications/Adobe Photoshop CS2/Presets/Scripts.
# If it isn't, you'll have to edit the IDIR line in this shell script below
# this block of comments.
#
# You must also have ImageMagick installed and working. This package can be
# found at http://imagemagick.org.
#
# Before going any further, verify that ContactSheet works in Basic and HP1
# mode correctly. Then verify that you can run ImageMagick by entering
#   % convert :logo /tmp/logo.gif
#
# To install the HP2Mac support, open a terminal window and change to the CSX
# installation directory: that's the '/csx' subfolder in your Presets/Scripts
# folder.
#
# Run this script:
#   % ./hp2mac.sh
#
# When this script runs, it creates some folders and files underneath
# '/csx' which allow ContactSheetX to call ImageMagick from within Photoshop.
# When the script is complete it displays some of configuration information
# that verifies that everything installed correctly.
#
# You are now ready for running CSX in HP2 mode. To test this, change modes in
# CSX either via the UI or the INI file. The difference in performance that
# can be acheived is hard to determine ahead of time. In general, you will
# see the greatest gains when processing large numbers of large images. In
# fact, for a small number of images, you may see a noticable slow down.
#
# You can run this script repeatedly with no ill effects.
# To undo its changes, delete the Presets/Scripts/csx/macexec.app folder and
# all of its contents.
#
# The only time you should need to re-run this script is if you upgrade or
#  move ImageMagick
#
# Change this if you have Photoshop CS2 installed somewhere else
IDIR="/Applications/Adobe Photoshop CS2/Presets/Scripts/csx"

if [ ! -e "$IDIR" ]; then
   echo Please unzip the CSX zip file in your Presets/Scripts directory.
   exit 1
fi

if [ "$MAGICK_HOME" == "" ]; then
    echo Missing MAGICK_HOME environment variable. Please install ImageMagick.
    exit 1
fi

# insert additional test to make sure that IM works
if [ ! -e "$MAGICK_HOME/bin/convert" ]; then
    echo There is a problem with your ImageMagick installation.
    echo Please reinstall ImageMagick and verify it before running this script.
    exit 1
fi

convert logo: /tmp/logo

cd "$IDIR"
mkdir -p macexec.app
mkdir -p macexec.app/Contents
rm -f macexec.app/Contents/macexec
touch macexec.app/Contents/macexec
chmod 755 macexec.app/Contents/macexec

cat > macexec.app/Contents/macexec.env << EOF
#!/bin/bash
export MAGICK_HOME=$MAGICK_HOME
export PATH="\${MAGICK_HOME}/bin:\${PATH}"
export DYLD_LIBRARY_PATH="\${MAGICK_HOME}/lib"
EOF

# EOT
